<?php
///////////////////////////////////////////////////////////////////////////

const ECODE_QUERY_LIMIT_EXCEEDED = 31;

class Ipnet
{
    public static $SERVER = 'tv.ipnet.ua';
}

const IPNET_LOGIN_URL =
    'http://%s/dune/api/login?login=%s&pass=%s&settings=all';

const IPNET_LAN_LOGIN_URL =
	//'http://%s/dune/api/login?login=123456789&pass=222&settings=all';
	'http://%s/dune/api/login?auto=1&settings=all';
	
const IPNET_GUEST_CHANNEL_LIST_URL =
    'http://%s/dune/api/channel_list';

const IPNET_CHANNEL_LIST_URL =
    'http://%s/dune/api/channel_list?%s=%s';

const IPNET_GUEST_EPG_URL =
    'http://%s/dune/api/epg?cid=%s&day=%s';

const IPNET_EPG_URL =
    'http://%s/dune/api/epg?%s=%s&cid=%s&day=%s';

const IPNET_GUEST_GET_URL_URL =
    'http://ts://%s/dune/api/get_url?cid=%s';

const IPNET_GET_URL_URL =
    'http://ts://%s/dune/api/get_url?%s=%s&cid=%s';

const IPNET_ARCHIVE_URL_PREFIX = 'http://tv.ipnet.ua/dune/data';

const IPNET_ARCHIVE_ID = 'main';

class IpnetSession
{
    private $http_opts = null;

    private $logged_in = false;
    private $guest_login = false;
    private $sid_name = null;
    private $sid = null;

    private $login_incorrect = false;

    private $account = null;
    private $services = null;
    private $settings = null;

    private $channel_list = null;

    private $unset_login_listener = null;

    private $has_multicast;

    ///////////////////////////////////////////////////////////////////////

    public function __construct($unset_login_listener)
    {
        $this->http_opts = array (
            CURLOPT_FOLLOWLOCATION  => true,
            CURLOPT_MAXREDIRS       => 10,
// debug            CURLOPT_PROXY           => '192.168.1.9:3128',
            CURLOPT_COOKIEFILE      => '/tmp/ipnet.cookies',
            CURLOPT_COOKIEJAR       => '/tmp/ipnet.cookies'
        );

        $this->unset_login_listener = $unset_login_listener;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_has_multicast()
    { return $this->has_multicast; }

    public function set_has_multicast($has_multicast)
    { $this->has_multicast = $has_multicast; }

    public function get_use_multicast(&$plugin_cookies)
    {
        $playback_mode = 'auto';
        if (isset($plugin_cookies->playback_mode))
            $playback_mode = $plugin_cookies->playback_mode;

        if ($playback_mode == 'multicast')
            return true;
        else if ($playback_mode == 'unicast')
            return false;
        else
            return $this->get_has_multicast();
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_channel_list()
    { return $this->channel_list; }

    public function get_account()
    { return $this->account; }

    public function get_settings()
    { return $this->settings; }

    ///////////////////////////////////////////////////////////////////////

    public function is_login_incorrect()
    { return $this->login_incorrect; }

    public function is_logged_in()
    { return $this->logged_in; }

    public function unset_login()
    {
        $this->logged_in = false;
        $this->guest_login = false;
        $this->sid_name = null;
        $this->sid = null;

        $this->account = null;
        $this->services = null;

        $this->settings = null;

        $this->channel_list = null;
    }

    public function logout()
    {
        $this->unset_login_listener->unset_login();
    }

    ///////////////////////////////////////////////////////////////////////

    private function fetch_document($url)
    {
        $doc = HD::http_get_document($url, $this->http_opts);

        // assert($doc != null).

        return $doc;
    }

    public function api_call($url, $silent=true)
    {
        for (;;)
        {
            $str = $this->fetch_document($url);

            if (!$silent)
                echo("Document:\n$str\n");

            $reply = simplexml_load_string($str);

            if ($reply === null)
                throw new Exception('Invalid data received from server');

            if (isset($reply->error))
            {
                $ecode = intval($reply->error->code);

                if ($ecode === ECODE_QUERY_LIMIT_EXCEEDED)
                {
                    hd_print('API: query limit exceeded');
                    continue;
                }

                hd_print("API error: URL '$url' " .
                    "returned error with code=$ecode, msg=" .
                    $reply->error->message . '.');

                throw new IpnetException(
                    $reply->error->message, $ecode);
            }

            break;
        }

        return $reply;
    }

    ///////////////////////////////////////////////////////////////////////

    public function dune_api_exception($e, $def_caption, $is_playback)
    {
        $is_ipnet_error = ($e instanceof IpnetException);
        if (!$is_ipnet_error)
        {
            hd_print('General exception: ' . $e->getMessage());
            $title = !$is_playback ? 'Error' :
                'Не удалось подключиться к Интернету.';
            $text_lines = $is_playback ? array() :
                array(
                 'Не удалось подключиться к Интернету.',
                 'Пожалуйста, проверьте подключение к Интернету.'
                );
            return new DuneException(
                $def_caption, -1,
                ActionFactory::show_error(false, $title, $text_lines));
        }
        hd_print('IP.NET API exception: ' .
            'code=' . $e->getCode() . ', message=' . $e->getMessage());

        $ecode = $e->getCode();
        $fatal = $ecode == 5 || $ecode == 6 ||
            $ecode == 11 || $ecode == 12 || $ecode == 13;

        if ($fatal)
            $this->logout();

        if ($ecode == 2 || $ecode == 4)
        {
            $this->login_incorrect = true;
            $this->logged_in = false;

            $title = !$is_playback ? 'Error' :
                'Недействительная подписка, пожалуйста, проверьте настройки.';
            $text_lines = $is_playback ? array() :
                array('Недействительная подписка, пожалуйста, проверьте настройки.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 3)
        {
            $title = !$is_playback ? 'Error' :
                'Доступ запрещен в течение 10 минут.';
            $text_lines = $is_playback ? array() :
                array('Доступ запрещен в течение 10 минут.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 12)
        {
            $title = !$is_playback ? 'Error' :
                'Отказано в доступе. Наверное подписка уже используется.';
            $text_lines = $is_playback ? array() :
                array('Отказано в доступе.',
                    'Наверное подписка уже используется.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 11)
        {
            $title = !$is_playback ? 'Error' :
                'Подписка уже используется.';
            $text_lines = $is_playback ? array() :
                array('Подписка уже используется.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 19)
        {
            $title = !$is_playback ? 'Error' :
                'Текущий код является недействительным.';
            $text_lines = $is_playback ? array() :
                array('Текущий код является недействительным.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        $title = !$is_playback ? 'Error' : $e->getMessage();
        $text_lines = $is_playback ? array() :
            array('Описание: ' . $e->getMessage(),
                'Код ошибки: ' . $ecode);
        return new DuneException(
            $def_caption, $ecode,
            ActionFactory::show_error($fatal, $title, $text_lines));
    }

    ///////////////////////////////////////////////////////////////////////

    public function api_get_stream_url($playback_url)
    {
        if (substr($playback_url, 0, 12) === 'http://ts://')
            $playback_url = 'http://' . substr($playback_url, 12);
        else if (substr($playback_url, 0, 13) === 'http://mp4://')
            $playback_url = 'http://' . substr($playback_url, 13);

        try
        {
            $doc = $this->api_call($playback_url);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        if (!isset($doc->url))
            throw new Exception('Get_url failed.');

        $len = strlen($doc->url);
        $pos = $len;
        for ($i = 10; $i < $len; $i++)
        {
            if ($doc->url[$i] == '"' || $doc->url[$i] == ' ')
            {
                $pos = $i;
                break;
            }
        }

        $url = substr($doc->url, 0, $pos);

        if (substr($url, 0, 10) === 'http/ts://')
            return 'http://' . substr($url, 10);

        return $url;
    }

    private function set_guest_login_state()
    {
        $this->logged_in = true;
        $this->guest_login = true;
        $this->sid = null;
        $this->sid_name = null;
        $this->login_incorrect = false;
        $this->account = null;
        $this->services = null;
        $this->settings = null;
    }

	private function api_lan_login()
    {
		try
		{
			$doc = $this->api_call(
				sprintf(
					IPNET_LAN_LOGIN_URL, Ipnet::$SERVER),
				false);
		}
		catch (Exception $e)
		{
			throw $this->dune_api_exception($e, 'Lan Login failed.', false);
		}
		
        $this->logged_in = true;
        $this->guest_login = false;
        $this->sid = strval($doc->sid);
        $this->sid_name = strval($doc->sid_name);
        $this->login_incorrect = false;
        $this->account = $doc->account;
        $this->services = $doc->services;
        $this->settings = $doc->settings;
    }
	
    private function api_login($user_name, $password)
    {
		try
		{
			$doc = $this->api_call(
				sprintf(
					IPNET_LOGIN_URL, Ipnet::$SERVER,
					$user_name, $password),
				false);
		}
		catch (Exception $e)
		{
			throw $this->dune_api_exception($e, 'Login failed.', false);
		}

        $this->logged_in = true;
        $this->guest_login = false;
        $this->sid = strval($doc->sid);
        $this->sid_name = strval($doc->sid_name);
        $this->login_incorrect = false;
        $this->account = $doc->account;
        $this->services = $doc->services;
        $this->settings = $doc->settings;
    }

    private function api_channel_list()
    {
        try
        {
            $doc = $this->api_call(
                sprintf(
                    IPNET_CHANNEL_LIST_URL, Ipnet::$SERVER,
                    $this->get_sid_name(), $this->get_sid()),
                true);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Channel list request failed.', false);
        }

        return $doc;
    }

    private function api_guest_channel_list()
    {
        try
        {
            $doc = $this->api_call(
                sprintf(
                    IPNET_GUEST_CHANNEL_LIST_URL, Ipnet::$SERVER),
                true);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Guest channel list request failed.', false);
        }

        return $doc;
    }

    ///////////////////////////////////////////////////////////////////////

    private function run_check_multicast($address, $port, $timeout_sec)
    {
        $plugin_dir = DuneSystem::$properties['install_dir_path'];

        hd_print("Running $plugin_dir/check_multicast for $address:$port ".
            "(timeout=${timeout_sec}s)");

			
		system("cp $plugin_dir/check_multicast /tmp/check_multicast", $return_var);
		system("chmod +x /tmp/check_multicast", $return_var);
		system("/tmp/check_multicast $address $port $timeout_sec", $return_var);
        //system("$plugin_dir/check_multicast $address $port $timeout_sec", $return_var);

        hd_print("Returned val = $return_var");

        return $return_var == 0;
    }

    private function check_multicast($channel_id, &$plugin_cookies)
    {
        /*$playback_url = sprintf(IPNET_GUEST_GET_URL_URL,
            Ipnet::$SERVER, $channel_id);

        $unicast_stream_url = $this->api_get_stream_url($playback_url);

        $pos1 = strpos($unicast_stream_url, '/udp/');
        if ($pos1 === false)
            throw new Exception("Unsupported stream url: $unicast_stream_url");
        $pos1 = $pos1 + 5;

        $pos2 = strpos($unicast_stream_url, ':', $pos1);
        if ($pos2 === false)
            throw new Exception("Unsupported stream url: $unicast_stream_url");
        $pos2 = $pos2 + 1;

        $pos3 = strpos($unicast_stream_url, '/', $pos2);
        if ($pos3 === false)
            throw new Exception("Unsupported stream url: $unicast_stream_url");

        $address = substr($unicast_stream_url, $pos1, $pos2 - $pos1 - 1);
        $port = intval(substr($unicast_stream_url, $pos2, $pos3 - $pos2));

        return $this->run_check_multicast($address, $port, 3);*/
        return false;
    }
	
    public function try_login(&$plugin_cookies)
    {
        // Check has_multicast.

        $tmp_dir_path = DuneSystem::$properties['tmp_dir_path'];
        $has_multicast_path = "$tmp_dir_path/has_multicast";
        if (!file_exists($has_multicast_path))
        {
            hd_print("Ipnet: has_multicast not cached yet.");

            $doc = $this->api_guest_channel_list();

            $channel_id = null;
            foreach ($doc->groups->item as $g)
            {
                foreach ($g->channels->item as $c)
                {
                    $channel_id = strval($c->id);
                    break;
                }
                if (!is_null($channel_id))
                    break;
            }

            if (is_null($channel_id))
                throw new Exception('Channel list is empty.');

            $has_multicast = $this->check_multicast($channel_id, $plugin_cookies);

            file_put_contents($has_multicast_path, intval($has_multicast));
        }
        else
            $has_multicast = intval(file_get_contents($has_multicast_path));

        $this->set_has_multicast($has_multicast);

        hd_print("Ipnet: has_multicast = " . ($has_multicast ? 'yes.' : 'no.'));

        // Perform login.

        if ($this->get_use_multicast(&$plugin_cookies))
        {
            $this->set_guest_login_state();

            $this->channel_list = $this->api_guest_channel_list();

            return;
        }
		
        //if (!isset($plugin_cookies->user_name))
		//    throw new Exception('User name is not set');

        try
        {
			if (!isset($plugin_cookies->user_name) ||
				($plugin_cookies->user_name === ''))
			{
				$this->api_lan_login();
				
				if ($this->is_logged_in())
					$this->channel_list = $this->api_channel_list();
			}
			else
			{
				$this->api_login(
					$plugin_cookies->user_name,
					$plugin_cookies->password);
					
				$this->channel_list = $this->api_channel_list();
			}
        }
        catch (Exception $e)
        {
            $this->logout();
            throw $e;
        }
    }

    public function ensure_logged_in(&$plugin_cookies)
    {
        if ($this->logged_in)
            return;

        $this->logout();

        $this->try_login($plugin_cookies);
    }

    public function check_logged_in()
    {
        if (!$this->logged_in)
            throw new IpnetException('Not logged in', 12);
    }

    public function is_guest_login()
    {
        return $this->logged_in && $this->guest_login;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_sid()
    {
        if (!$this->logged_in)
            throw new IpnetException('Not logged in', 12);
        return $this->sid;
    }

    public function get_sid_name()
    {
        if (!$this->logged_in)
            throw new IpnetException('Not logged in', 12);
        return $this->sid_name;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_icon($id)
    {
        $archive = DefaultArchive::get_archive(
            IPNET_ARCHIVE_ID,
            IPNET_ARCHIVE_URL_PREFIX);

        return $archive->get_archive_url($id);
    }

    public function get_group_icon($group_id)
    {
        return $this->get_icon("group_$group_id.png");
    }

    public function get_channel_icon($channel_id)
    {
        return $this->get_icon("channel_$channel_id.png");
    }

    ///////////////////////////////////////////////////////////////////////

    private function do_get_edit_subscription_defs(
        &$plugin_cookies, $handler, $add_params)
    {
        $defs = array();

        $user_name = isset($plugin_cookies->user_name) ?
            $plugin_cookies->user_name : '';
        $password = isset($plugin_cookies->password) ?
            $plugin_cookies->password : '';

        ControlFactory::add_text_field($defs,
            $handler, $add_params,
            'user_name', 'Подписка:',
            $user_name, true, false, false, false, 500);

        ControlFactory::add_text_field($defs,
            $handler, $add_params,
            'password', 'Пароль:',
            $password, true, true, false, false, 500);

        ControlFactory::add_vgap($defs, 50);

        ControlFactory::add_button($defs,
            $handler, $add_params,
            'apply_subscription', null, 'Применить', 300);

        ControlFactory::add_vgap($defs, -3);

        ControlFactory::add_close_dialog_button($defs,
            'Отменить', 300);

        return $defs;
    }

    public function do_get_edit_subscription_action(
        &$plugin_cookies, $handler, $add_params = null)
    {
        return ActionFactory::show_dialog(
            'Введите данные IP.NET подписки',
            $this->do_get_edit_subscription_defs(
                $plugin_cookies, $handler, $add_params),
            true);
    }

    public function apply_subscription(&$plugin_cookies, &$user_input)
    {
        if (!isset($user_input->control_id) ||
            $user_input->control_id != 'apply_subscription')
        {
            return false;
        }

        if ($user_input->user_name === '')
        {
            return array(
                'need_close_dialog' => false,
                'action' =>
                    ActionFactory::show_error(false,
                        'Error',
                        array('Поле Подписка не может быть пустым.')));
        }

        $plugin_cookies->user_name = $user_input->user_name;
        $plugin_cookies->password = $user_input->password;

        $this->logout();

        $post_action = null;
        try
        {
            $this->try_login($plugin_cookies);
        }
        catch (DuneException $e)
        {
            $post_action = $e->get_error_action();
        }

        return array(
            'need_close_dialog' => true,
            'action' => $post_action);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_archive()
    {
        return DefaultArchive::get_archive(
            IPNET_ARCHIVE_ID, IPNET_ARCHIVE_URL_PREFIX);
    }
}

///////////////////////////////////////////////////////////////////////////
?>
