<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_controls_screen.php';

///////////////////////////////////////////////////////////////////////////

class IpnetSetupScreen extends AbstractControlsScreen
{
    const ID = 'setup';
	const VERSION = '0.0.3';

    ///////////////////////////////////////////////////////////////////////

    private $session;
    private $tv;

    ///////////////////////////////////////////////////////////////////////

    public function __construct($session, Tv $tv)
    {
        parent::__construct(self::ID);

        $this->session = $session;
        $this->tv = $tv;
    }

    private function do_get_control_defs(&$plugin_cookies)
    {
        $defs = array();

        // Show in main screen.
		
        $this->add_label($defs,
        'Версия:', self::VERSION);

        if (isset($plugin_cookies->show_in_main_screen))
            $show_in_main_screen = $plugin_cookies->show_in_main_screen;
        else
            $show_in_main_screen = 'auto';

        $show_ops['auto'] = 'Авто';
        $show_ops['yes'] = 'Да';
        $show_ops['no'] = 'Нет';
        $this->add_combobox($defs,
            'show_in_main_screen', 'Показывать на главном экране:',
            $show_in_main_screen, $show_ops, 0, true);

        ///////////////////////////////////////////////////////////////////////
        // Playback mode.

        if (isset($plugin_cookies->playback_mode))
            $playback_mode = $plugin_cookies->playback_mode;
        else
            $playback_mode = 'auto';

        $playback_mode_ops['auto'] = 'Авто';
        $playback_mode_ops['multicast'] = 'Multicast (локальная сеть)';
        $playback_mode_ops['unicast'] = 'Unicast (Интернет)';

        $this->add_combobox($defs,
            'playback_mode', 'Режим воспроизведения:',
            $playback_mode, $playback_mode_ops, 0, true);

        ///////////////////////////////////////////////////////////////////////
        // UseMulticast-dependent fields.

        $use_multicast = $this->session->get_use_multicast($plugin_cookies);

        if ($use_multicast)
        {
            $this->add_label($defs,
                'Статус:', 'Локальная сеть');
        }
        else
        {
            $user_name = isset($plugin_cookies->user_name) ?
                $plugin_cookies->user_name : '';
            $password = isset($plugin_cookies->password) ?
                $plugin_cookies->password : '';

            $logged_in = $this->session->is_logged_in();

            hd_print('Packet name: ' .
                ($logged_in ? $this->session->get_account()->packet_name : 'unset'));
            if ($user_name === '')
                $login_str = 'unset';
            else if (!$logged_in ||
                !isset($this->session->get_account()->packet_name))
            {
                $login_str = $user_name;
            }
            else
            {
                $login_str = $user_name . ' (' .
                    $this->session->get_account()->packet_name . ')';
            }

            hd_print('Packet expire: ' .
                ($logged_in ? $this->session->get_account()->packet_expire : 'unset'));
            if (!$logged_in ||
                !isset($this->session->get_account()->packet_expire) ||
                $this->session->get_account()->packet_expire <= 0)
            {
                $expires_str = 'недоступно';
            }
            else
            {
                $tm = intval($this->session->get_account()->packet_expire);

                $expires_str =
                    HD::format_date_time_date($tm) . ', ' .
                    HD::format_date_time_time($tm);
            }

            $this->add_label($defs,
                'Статус:',
                $logged_in ?
                    'Интернет (вход выполнен)' : 'Интернет (вход не выполнен)');

            $this->add_label($defs,
                'Абонемент:', $login_str);

            $this->add_label($defs,
                'Истекает:', $expires_str);

            $this->add_button($defs, 'edit_subscription', null,
                'Редактировать абонемент...', 0);
        }

        return $defs;
    }

    public function get_control_defs(MediaURL $media_url, &$plugin_cookies)
    {
        try
        {
            if (!$this->session->is_login_incorrect())
                $this->session->ensure_logged_in($plugin_cookies);

            $this->tv->ensure_channels_loaded($plugin_cookies);
        }
        catch (Exception $e)
        {
            // Nop.
        }

        return $this->do_get_control_defs($plugin_cookies);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('Setup: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $need_close_dialog = false;
        $need_reset_controls = false;
        $post_action = null;
        if ($user_input->action_type === 'apply')
        {
            $control_id = $user_input->control_id;
            if ($control_id === 'edit_subscription')
            {
                return $this->session->do_get_edit_subscription_action(
                    $plugin_cookies, $this);
            }
            else if ($control_id === 'apply_subscription')
            {
                if ($user_input->user_name === '')
                {
                    return ActionFactory::show_error(false,
                        'Error',
                        array('Поле Подписка не может быть пустым.'));
                }

                $plugin_cookies->user_name = $user_input->user_name;
                $plugin_cookies->password = $user_input->password;

                $this->session->logout();

                try
                {
                    $this->session->try_login($plugin_cookies);
                }
                catch (DuneException $e)
                {
                    $post_action = $e->get_error_action();
                }

                $need_close_dialog = true;
                $need_reset_controls = true;
            }
        }
        else if ($user_input->action_type === 'confirm')
        {
            $control_id = $user_input->control_id;
            $new_value = $user_input->{$control_id};
            hd_print("Setup: changing $control_id value to $new_value");

            if ($control_id === 'show_in_main_screen')
            {
                $plugin_cookies->show_in_main_screen = $new_value;
                if ($new_value === 'auto')
                    $plugin_cookies->show_tv = 'lang(russian,ukrainian)';
                else
                    $plugin_cookies->show_tv = $new_value;
            }
            else if ($control_id === 'playback_mode')
            {
                $old_use_multicast =
                    $this->session->get_use_multicast($plugin_cookies);

                $plugin_cookies->playback_mode = $new_value;

                $new_use_multicast =
                    $this->session->get_use_multicast($plugin_cookies);

                $this->session->logout();
                if ($new_use_multicast && !$old_use_multicast)
                {
                    $this->session->try_login($plugin_cookies);
                }
                else if (!$new_use_multicast && $old_use_multicast)
                {
					try
					{
						$this->session->try_login($plugin_cookies);
					}
					catch (Exception $e)
					{
						return ActionFactory::reset_controls(
							$this->do_get_control_defs($plugin_cookies),
							$this->session->do_get_edit_subscription_action(
								$plugin_cookies, $this));
					}
                }
            }
            else
                return null;

            $need_reset_controls = true;
        }

        if ($need_reset_controls)
        {
            $defs = $this->do_get_control_defs($plugin_cookies);

            $reset_controls_action = ActionFactory::reset_controls(
                $defs, $post_action);

            if ($need_close_dialog)
            {
                return ActionFactory::close_dialog_and_run(
                    $reset_controls_action);
            }

            return $reset_controls_action;
        }

        return null;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
